@@ -918,13 +918,17 @@ class ConnectionSendThread : public JThread {
 public:
 	friend class UDPPeer;
 
-	ConnectionSendThread(Connection* parent,
-							unsigned int max_packet_size, float timeout);
+	ConnectionSendThread(unsigned int max_packet_size, float timeout);
 
 	void * Thread       ();
 
 	void Trigger();
 
+	void setParent(Connection* parent) {
+		assert(parent != NULL);
+		m_connection = parent;
+	}
+
 	void setPeerTimeout(float peer_timeout)
 		{ m_timeout = peer_timeout; }
 
@@ -970,11 +974,15 @@ private:
 
 class ConnectionReceiveThread : public JThread {
 public:
-	ConnectionReceiveThread(Connection* parent,
-							unsigned int max_packet_size);
+	ConnectionReceiveThread(unsigned int max_packet_size);
 
 	void * Thread       ();
 
+	void setParent(Connection* parent) {
+		assert(parent != NULL);
+		m_connection = parent;
+	}
+
 private:
 	void receive        ();
 
